@@ -54,6 +54,8 @@ impl Choice {
     /// This function only exists as an escape hatch for the rare case
     /// where it's not possible to use one of the `subtle`-provided
     /// trait impls.
+    ///
+    /// To convert a `Choice` to a `bool`, use the `From` implementation instead.
     #[inline]
     pub fn unwrap_u8(&self) -> u8 {
         self.0
@@ -76,13 +78,8 @@ impl From<Choice> for bool {
     /// verification.
     #[inline]
     fn from(source: Choice) -> bool {
-        debug_assert!(source.0 == 0u8 || source.0 == 1u8);
-
-        match source.0 {
-            0 => return false,
-            1 => return true,
-            _ => unsafe { ::core::hint::unreachable_unchecked() },
-        }
+        debug_assert!((source.0 == 0u8) | (source.0 == 1u8));
+        source.0 != 0
     }
 }
 
@@ -144,12 +141,9 @@ impl Not for Choice {
 /// than its type.
 ///
 /// Uses inline asm when available, otherwise it's a no-op.
-#[cfg(all(
-    feature = "nightly",
-    not(any(target_arch = "asmjs", target_arch = "wasm32"))
-))]
+#[cfg(all(feature = "nightly", not(any(target_arch = "asmjs", target_arch = "wasm32"))))]
 fn black_box(input: u8) -> u8 {
-    debug_assert!(input == 0u8 || input == 1u8);
+    debug_assert!((input == 0u8) | (input == 1u8));
 
     // Pretend to access a register containing the input.  We "volatile" here
     // because some optimisers treat assembly templates without output operands
@@ -158,14 +152,10 @@ fn black_box(input: u8) -> u8 {
 
     input
 }
-#[cfg(any(
-    target_arch = "asmjs",
-    target_arch = "wasm32",
-    not(feature = "nightly")
-))]
+#[cfg(any(target_arch = "asmjs", target_arch = "wasm32", not(feature = "nightly")))]
 #[inline(never)]
 fn black_box(input: u8) -> u8 {
-    debug_assert!(input == 0u8 || input == 1u8);
+    debug_assert!((input == 0u8) | (input == 1u8));
     // We don't have access to inline assembly or test::black_box or ...
     //
     // Bailing out, hopefully the compiler doesn't use the fact that `input` is 0 or 1.
@@ -259,24 +249,15 @@ macro_rules! generate_integer_equal {
         impl ConstantTimeEq for $t_u {
             #[inline]
             fn ct_eq(&self, other: &$t_u) -> Choice {
-                // First construct x such that self == other iff all bits of x are 1
-                let mut x: $t_u = !(self ^ other);
+                // x == 0 if and only if self == other
+                let x: $t_u = self ^ other;
 
-                // Now compute the and of all bits of x.
-                //
-                // e.g. for a u8, do:
-                //
-                //    x &= x >> 4;
-                //    x &= x >> 2;
-                //    x &= x >> 1;
-                //
-                let mut shift: usize = $bit_width / 2;
-                while shift >= 1 {
-                    x &= x >> shift;
-                    shift /= 2;
-                }
+                // If x == 0, then x and -x are both equal to zero;
+                // otherwise, one or both will have its high bit set.
+                let y: $t_u = (x | x.wrapping_neg()) >> ($bit_width - 1);
 
-                (x as u8).into()
+                // Result is the opposite of the high bit (now shifted to low).
+                ((y ^ (1 as $t_u)) as u8).into()
             }
         }
         impl ConstantTimeEq for $t_i {
@@ -293,24 +274,15 @@ generate_integer_equal!(u8, i8, 8);
 generate_integer_equal!(u16, i16, 16);
 generate_integer_equal!(u32, i32, 32);
 generate_integer_equal!(u64, i64, 64);
+#[cfg(feature = "i128")]
 generate_integer_equal!(u128, i128, 128);
 generate_integer_equal!(usize, isize, ::core::mem::size_of::<usize>() * 8);
 
-/// Select one of two inputs according to a `Choice` in constant time.
+/// A type which can be conditionally selected in constant time.
 ///
-/// # Examples
-///
-/// ```
-/// # use subtle;
-/// use subtle::ConditionallySelectable;
-/// use subtle::Choice;
-/// let a: i32 = 5;
-/// let b: i32 = 13;
-///
-/// assert_eq!(i32::conditional_select(&a, &b, Choice::from(0)), a);
-/// assert_eq!(i32::conditional_select(&a, &b, Choice::from(1)), b);
-/// ```
-pub trait ConditionallySelectable {
+/// This trait also provides generic implementations of conditional
+/// assignment and conditional swaps.
+pub trait ConditionallySelectable: Copy {
     /// Select `a` or `b` according to `choice`.
     ///
     /// # Returns
@@ -319,8 +291,80 @@ pub trait ConditionallySelectable {
     /// * `b` if `choice == Choice(1)`.
     ///
     /// This function should execute in constant time.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # extern crate subtle;
+    /// use subtle::ConditionallySelectable;
+    /// #
+    /// # fn main() {
+    /// let x: u8 = 13;
+    /// let y: u8 = 42;
+    ///
+    /// let z = u8::conditional_select(&x, &y, 0.into());
+    /// assert_eq!(z, x);
+    /// let z = u8::conditional_select(&x, &y, 1.into());
+    /// assert_eq!(z, y);
+    /// # }
+    /// ```
     #[inline]
     fn conditional_select(a: &Self, b: &Self, choice: Choice) -> Self;
+
+    /// Conditionally assign `other` to `self`, according to `choice`.
+    ///
+    /// This function should execute in constant time.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # extern crate subtle;
+    /// use subtle::ConditionallySelectable;
+    /// #
+    /// # fn main() {
+    /// let mut x: u8 = 13;
+    /// let mut y: u8 = 42;
+    ///
+    /// x.conditional_assign(&y, 0.into());
+    /// assert_eq!(x, 13);
+    /// x.conditional_assign(&y, 1.into());
+    /// assert_eq!(x, 42);
+    /// # }
+    /// ```
+    #[inline]
+    fn conditional_assign(&mut self, other: &Self, choice: Choice) {
+        *self = Self::conditional_select(self, other, choice);
+    }
+
+    /// Conditionally swap `self` and `other` if `choice == 1`; otherwise,
+    /// reassign both unto themselves.
+    ///
+    /// This function should execute in constant time.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # extern crate subtle;
+    /// use subtle::ConditionallySelectable;
+    /// #
+    /// # fn main() {
+    /// let mut x: u8 = 13;
+    /// let mut y: u8 = 42;
+    ///
+    /// u8::conditional_swap(&mut x, &mut y, 0.into());
+    /// assert_eq!(x, 13);
+    /// assert_eq!(y, 42);
+    /// u8::conditional_swap(&mut x, &mut y, 1.into());
+    /// assert_eq!(x, 42);
+    /// assert_eq!(y, 13);
+    /// # }
+    /// ```
+    #[inline]
+    fn conditional_swap(a: &mut Self, b: &mut Self, choice: Choice) {
+        let t: Self = *a;
+        a.conditional_assign(&b, choice);
+        b.conditional_assign(&t, choice);
+    }
 }
 
 macro_rules! to_signed_int {
@@ -364,7 +408,25 @@ macro_rules! generate_integer_conditional_select {
                 // if choice = 0, mask = (-0) = 0000...0000
                 // if choice = 1, mask = (-1) = 1111...1111
                 let mask = -(choice.unwrap_u8() as to_signed_int!($t)) as $t;
-                a ^ ((mask) & (a ^ b))
+                a ^ (mask & (a ^ b))
+            }
+
+            #[inline]
+            fn conditional_assign(&mut self, other: &Self, choice: Choice) {
+                // if choice = 0, mask = (-0) = 0000...0000
+                // if choice = 1, mask = (-1) = 1111...1111
+                let mask = -(choice.unwrap_u8() as to_signed_int!($t)) as $t;
+                *self ^= mask & (*self ^ *other);
+            }
+
+            #[inline]
+            fn conditional_swap(a: &mut Self, b: &mut Self, choice: Choice) {
+                // if choice = 0, mask = (-0) = 0000...0000
+                // if choice = 1, mask = (-1) = 1111...1111
+                let mask = -(choice.unwrap_u8() as to_signed_int!($t)) as $t;
+                let t = mask & (*a ^ *b);
+                *a ^= t;
+                *b ^= t;
             }
          }
     )*)
@@ -374,14 +436,23 @@ generate_integer_conditional_select!(  u8   i8);
 generate_integer_conditional_select!( u16  i16);
 generate_integer_conditional_select!( u32  i32);
 generate_integer_conditional_select!( u64  i64);
+#[cfg(feature = "i128")]
 generate_integer_conditional_select!(u128 i128);
 
+impl ConditionallySelectable for Choice {
+    #[inline]
+    fn conditional_select(a: &Self, b: &Self, choice: Choice) -> Self {
+        Choice(u8::conditional_select(&a.0, &b.0, choice))
+    }
+}
+
 /// A type which can be conditionally negated in constant time.
 ///
 /// # Note
 ///
-/// A generic implementation of `ConditionallyNegatable` is provided for types
-/// which are `ConditionallyNegatable + Neg`.
+/// A generic implementation of `ConditionallyNegatable` is provided
+/// for types `T` which are `ConditionallySelectable` and have `Neg`
+/// implemented on `&T`.
 pub trait ConditionallyNegatable {
     /// Negate `self` if `choice == Choice(1)`; otherwise, leave it
     /// unchanged.
@@ -393,7 +464,7 @@ pub trait ConditionallyNegatable {
 
 impl<T> ConditionallyNegatable for T
 where
-    T: ConditionallyAssignable,
+    T: ConditionallySelectable,
     for<'a> &'a T: Neg<Output = T>,
 {
     #[inline]
@@ -404,171 +475,149 @@ where
     }
 }
 
-/// A type which can be conditionally assigned in constant time.
-pub trait ConditionallyAssignable {
-    /// Conditionally assign `other` to `self`, according to `choice`.
-    ///
-    /// This function should execute in constant time.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// # extern crate subtle;
-    /// use subtle::ConditionallyAssignable;
-    /// #
-    /// # fn main() {
-    /// let mut x: u8 = 13;
-    /// let y:     u8 = 42;
-    ///
-    /// x.conditional_assign(&y, 0.into());
-    /// assert_eq!(x, 13);
-    /// x.conditional_assign(&y, 1.into());
-    /// assert_eq!(x, 42);
-    /// # }
-    /// ```
-    ///
+/// The `CtOption<T>` type represents an optional value similar to the
+/// [`Option<T>`](core::option::Option) type but is intended for
+/// use in constant time APIs.
+///
+/// Any given `CtOption<T>` is either `Some` or `None`, but unlike
+/// `Option<T>` these variants are not exposed. The
+/// [`is_some()`](CtOption::is_some) method is used to determine if
+/// the value is `Some`, and [`unwrap_or()`](CtOption::unwrap_or) and
+/// [`unwrap_or_else()`](CtOption::unwrap_or_else) methods are
+/// provided to access the underlying value. The value can also be
+/// obtained with [`unwrap()`](CtOption::unwrap) but this will panic
+/// if it is `None`.
+///
+/// Functions that are intended to be constant time may not produce
+/// valid results for all inputs, such as square root and inversion
+/// operations in finite field arithmetic. Returning an `Option<T>`
+/// from these functions makes it difficult for the caller to reason
+/// about the result in constant time, and returning an incorrect
+/// value burdens the caller and increases the chance of bugs.
+#[derive(Clone, Copy, Debug)]
+pub struct CtOption<T> {
+    value: T,
+    is_some: Choice,
+}
+
+impl<T> CtOption<T> {
+    /// This method is used to construct a new `CtOption<T>` and takes
+    /// a value of type `T`, and a `Choice` that determines whether
+    /// the optional value should be `Some` or not. If `is_some` is
+    /// false, the value will still be stored but its value is never
+    /// exposed.
     #[inline]
-    fn conditional_assign(&mut self, other: &Self, choice: Choice);
-}
-
-impl<T> ConditionallyAssignable for T
-where
-    T: ConditionallySelectable,
-{
-    #[inline]
-    fn conditional_assign(&mut self, other: &Self, choice: Choice) {
-        *self = T::conditional_select(self, other, choice);
+    pub fn new(value: T, is_some: Choice) -> CtOption<T> {
+        CtOption { value: value, is_some: is_some }
     }
-}
 
-/// A type which is conditionally swappable in constant time.
-pub trait ConditionallySwappable {
-    /// Conditionally swap `self` and `other` if `choice == 1`; otherwise,
-    /// reassign both unto themselves.
+    /// This returns the underlying value but panics if it
+    /// is not `Some`.
+    #[inline]
+    pub fn unwrap(self) -> T {
+        assert_eq!(self.is_some.unwrap_u8(), 1);
+
+        self.value
+    }
+
+    /// This returns the underlying value if it is `Some`
+    /// or the provided value otherwise.
+    #[inline]
+    pub fn unwrap_or(self, def: T) -> T
+    where
+        T: ConditionallySelectable,
+    {
+        T::conditional_select(&def, &self.value, self.is_some)
+    }
+
+    /// This returns the underlying value if it is `Some`
+    /// or the value produced by the provided closure otherwise.
+    #[inline]
+    pub fn unwrap_or_else<F>(self, f: F) -> T
+    where
+        T: ConditionallySelectable,
+        F: FnOnce() -> T,
+    {
+        T::conditional_select(&f(), &self.value, self.is_some)
+    }
+
+    /// Returns a true `Choice` if this value is `Some`.
+    #[inline]
+    pub fn is_some(&self) -> Choice {
+        self.is_some
+    }
+
+    /// Returns a true `Choice` if this value is `None`.
+    #[inline]
+    pub fn is_none(&self) -> Choice {
+        !self.is_some
+    }
+
+    /// Returns a `None` value if the option is `None`, otherwise
+    /// returns a `CtOption` enclosing the value of the provided closure.
+    /// The closure is given the enclosed value or, if the option is
+    /// `None`, it is provided a dummy value computed using
+    /// `Default::default()`.
     ///
-    /// # Note
+    /// This operates in constant time, because the provided closure
+    /// is always called.
+    #[inline]
+    pub fn map<U, F>(self, f: F) -> CtOption<U>
+    where
+        T: Default + ConditionallySelectable,
+        F: FnOnce(T) -> U,
+    {
+        CtOption::new(
+            f(T::conditional_select(
+                &T::default(),
+                &self.value,
+                self.is_some,
+            )),
+            self.is_some,
+        )
+    }
+
+    /// Returns a `None` value if the option is `None`, otherwise
+    /// returns the result of the provided closure. The closure is
+    /// given the enclosed value or, if the option is `None`, it
+    /// is provided a dummy value computed using `Default::default()`.
     ///
-    /// This trait is generically implemented for any type which implements
-    /// `ConditionallyAssignable + Copy`.
+    /// This operates in constant time, because the provided closure
+    /// is always called.
     #[inline]
-    fn conditional_swap(&mut self, other: &mut Self, choice: Choice);
+    pub fn and_then<U, F>(self, f: F) -> CtOption<U>
+    where
+        T: Default + ConditionallySelectable,
+        F: FnOnce(T) -> CtOption<U>,
+    {
+        let mut tmp = f(T::conditional_select(
+            &T::default(),
+            &self.value,
+            self.is_some,
+        ));
+        tmp.is_some &= self.is_some;
+
+        tmp
+    }
 }
 
-impl<T> ConditionallySwappable for T
-where
-    T: ConditionallyAssignable + Copy,
-{
+impl<T: ConditionallySelectable> ConditionallySelectable for CtOption<T> {
+    fn conditional_select(a: &Self, b: &Self, choice: Choice) -> Self {
+        CtOption::new(
+            T::conditional_select(&a.value, &b.value, choice),
+            Choice::conditional_select(&a.is_some, &b.is_some, choice),
+        )
+    }
+}
+
+impl<T: ConstantTimeEq> ConstantTimeEq for CtOption<T> {
+    /// Two `CtOption<T>`s are equal if they are both `Some` and
+    /// their values are equal, or both `None`.
     #[inline]
-    fn conditional_swap(&mut self, other: &mut T, choice: Choice) {
-        let temp: T = *self;
-        self.conditional_assign(&other, choice);
-        other.conditional_assign(&temp, choice);
-    }
-}
+    fn ct_eq(&self, rhs: &CtOption<T>) -> Choice {
+        let a = self.is_some();
+        let b = rhs.is_some();
 
-#[cfg(test)]
-mod test {
-    use super::*;
-
-    #[test]
-    #[should_panic]
-    fn slices_equal_different_lengths() {
-        let a: [u8; 3] = [0, 0, 0];
-        let b: [u8; 4] = [0, 0, 0, 0];
-
-        assert_eq!((&a).ct_eq(&b).unwrap_u8(), 1);
-    }
-
-    #[test]
-    fn slices_equal() {
-        let a: [u8; 8] = [1, 2, 3, 4, 5, 6, 7, 8];
-        let b: [u8; 8] = [1, 2, 3, 4, 4, 3, 2, 1];
-
-        let a_eq_a = (&a).ct_eq(&a);
-        let a_eq_b = (&a).ct_eq(&b);
-
-        assert_eq!(a_eq_a.unwrap_u8(), 1);
-        assert_eq!(a_eq_b.unwrap_u8(), 0);
-
-        let c: [u8; 16] = [0u8; 16];
-
-        let a_eq_c = (&a).ct_eq(&c);
-        assert_eq!(a_eq_c.unwrap_u8(), 0);
-    }
-
-    #[test]
-    fn conditional_assign_i32() {
-        let mut a: i32 = 5;
-        let b: i32 = 13;
-
-        a.conditional_assign(&b, 0.into());
-        assert_eq!(a, 5);
-        a.conditional_assign(&b, 1.into());
-        assert_eq!(a, 13);
-    }
-
-    #[test]
-    fn conditional_assign_i64() {
-        let mut c: i64 = 2343249123;
-        let d: i64 = 8723884895;
-
-        c.conditional_assign(&d, 0.into());
-        assert_eq!(c, 2343249123);
-        c.conditional_assign(&d, 1.into());
-        assert_eq!(c, 8723884895);
-    }
-
-    macro_rules! generate_integer_conditional_select_tests {
-        ($($t:ty)*) => ($(
-            let x: $t = 0;  // all 0 bits
-            let y: $t = !0; // all 1 bits
-
-            assert_eq!(<$t>::conditional_select(&x, &y, 0.into()), 0);
-            assert_eq!(<$t>::conditional_select(&x, &y, 1.into()), y);
-        )*)
-    }
-
-    #[test]
-    fn integer_conditional_select() {
-        generate_integer_conditional_select_tests!(u8 u16 u32 u64 u128);
-        generate_integer_conditional_select_tests!(i8 i16 i32 i64 i128);
-    }
-
-    #[test]
-    fn custom_conditional_select_i16() {
-        let x: i16 = 257;
-        let y: i16 = 514;
-
-        assert_eq!(i16::conditional_select(&x, &y, 0.into()), 257);
-        assert_eq!(i16::conditional_select(&x, &y, 1.into()), 514);
-    }
-
-    macro_rules! generate_integer_equal_tests {
-        ($($t:ty),*) => ($(
-            let y: $t = 0;  // all 0 bits
-            let z: $t = !0; // all 1 bits
-
-            let x = z;
-
-            assert_eq!(x.ct_eq(&y).unwrap_u8(), 0);
-            assert_eq!(x.ct_eq(&z).unwrap_u8(), 1);
-        )*)
-    }
-
-    #[test]
-    fn integer_equal() {
-        generate_integer_equal_tests!(u8, u16, u32, u64, u128, usize);
-        generate_integer_equal_tests!(i8, i16, i32, i64, i128, isize);
-    }
-
-    #[test]
-    fn choice_into_bool() {
-        let choice_true: bool = Choice::from(1).into();
-
-        assert!(choice_true);
-
-        let choice_false: bool = Choice::from(0).into();
-
-        assert!(!choice_false);
+        (a & b & self.value.ct_eq(&rhs.value)) | (!a & !b)
     }
 }
